@@ -1,20 +1,29 @@
-/* comploader.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -22,11 +31,14 @@
 #include <tnt/componentfactory.h>
 #include <tnt/tntconfig.h>
 #include <tnt/httperror.h>
+#include <tnt/util.h>
 #include <cxxtools/log.h>
+#include <cxxtools/dlloader.h>
+#include <stdlib.h>
 
 namespace
 {
-  cxxtools::Mutex mutex;
+  cxxtools::ReadWriteMutex mutex;
 }
 
 namespace tnt
@@ -37,11 +49,54 @@ namespace tnt
 //
 log_define("tntnet.comploader")
 
-ComponentLibrary::~ComponentLibrary()
+void* ComponentLibrary::dlopen(const std::string& name, bool local)
 {
-  for (langlibsType::iterator it = langlibs.begin();
-       it != langlibs.end(); ++it)
-    delete it->second;
+  log_debug("dlopen <" << name << ">, " << local);
+
+  int flags = local ? RTLD_NOW|RTLD_LOCAL: RTLD_NOW|RTLD_GLOBAL;
+  std::string n = name;
+  if (!n.empty() && n[0] == '!')
+  {
+    flags = RTLD_NOW|RTLD_GLOBAL;
+    n.erase(0, 1);
+    log_debug("dlopen => <" << n << '>');
+  }
+
+  void* ret = ::dlopen((n + ".so").c_str(), flags);
+  if (ret != 0)
+  {
+    log_info("library \"" << n << ".so\"");
+    return ret;
+  }
+
+  ret = ::dlopen((n + ".a").c_str(), flags);
+  if (ret != 0)
+  {
+    log_info("library \"" << n << ".a\"");
+    return ret;
+  }
+
+  ret = ::dlopen((n + ".dll").c_str(), flags);
+  if (ret != 0)
+  {
+    log_info("library \"" << n << ".dll\"");
+    return ret;
+  }
+
+  ret = ::dlopen(n.c_str(), flags);
+  if (ret == 0)
+    log_warn("failed to load library \"" << n << '"');
+  else
+    log_info("library \"" << n << "\"");
+
+  return ret;
+}
+
+void ComponentLibrary::init(const std::string& name, bool local)
+{
+  void* handle = dlopen(name, local);
+  if (handle)
+    handlePtr = new HandleType(handle);
 }
 
 Component* ComponentLibrary::create(
@@ -66,14 +121,17 @@ Component* ComponentLibrary::create(
   return factory->create(ci, rootmapper, cl);
 }
 
-LangLib* ComponentLibrary::getLangLib(const std::string& lang)
+LangLib::PtrType ComponentLibrary::getLangLib(const std::string& lang)
 {
-  cxxtools::MutexLock lock(mutex);
+  cxxtools::ReadLock rlock(mutex);
   langlibsType::const_iterator it = langlibs.find(lang);
   if (it != langlibs.end())
     return it->second;
 
-  LangLib* l = 0;
+  rlock.unlock();
+  cxxtools::WriteLock wlock(mutex);
+
+  LangLib::PtrType l;
   try
   {
     std::string n = (path.empty() ? libname : (path + '/' + libname));
@@ -101,7 +159,7 @@ Comploader::Comploader()
          it != configLoad.end(); ++it)
     {
       if (it->params.empty())
-        throw std::runtime_error("missing libraryname in Load-command");
+        throwRuntimeError("missing libraryname in Load-command");
       fetchLib(it->params[0]);
     }
   }
@@ -123,20 +181,27 @@ Component& Comploader::fetchComp(const Compident& ci,
 {
   log_debug("fetchComp \"" << ci << '"');
 
-  cxxtools::MutexLock lock(mutex);
+  cxxtools::ReadLock rlock(mutex);
+  cxxtools::WriteLock wlock(mutex, false);
 
   // lookup Component
   componentmap_type::iterator it = componentmap.find(ci);
   if (it == componentmap.end())
   {
-    ComponentLibrary& lib = fetchLib(ci.libname);
-    Component* comp = lib.create(ci.compname, *this, rootmapper);
+    rlock.unlock();
+    wlock.lock();
+    it = componentmap.find(ci);
+    if (it == componentmap.end())
+    {
+      ComponentLibrary& lib = fetchLib(ci.libname);
+      Component* comp = lib.create(ci.compname, *this, rootmapper);
 
-    componentmap[ci] = comp;
-    return *comp;
+      componentmap[ci] = comp;
+      return *comp;
+    }
   }
-  else
-    return *(it->second);
+
+  return *(it->second);
 }
 
 Component* Comploader::createComp(const Compident& ci,
@@ -154,7 +219,7 @@ const char* Comploader::getLangData(const Compident& ci,
 {
   log_debug("getLangData(" << ci << ", \"" << lang << "\")");
   ComponentLibrary& lib = fetchLib(ci.libname);
-  LangLib* langLib = lib.getLangLib(lang);
+  LangLib::PtrType langLib = lib.getLangLib(lang);
   if (langLib)
     return langLib->getData(ci.compname);
   else
@@ -183,8 +248,16 @@ ComponentLibrary& Comploader::fetchLib(const std::string& libname)
 {
   log_debug("fetchLib \"" << libname << '"');
 
+  std::string n = libname;
+  bool local = false;
+  if (!n.empty() && n[0] == '!')
+  {
+    local = true;
+    n.erase(0, 1);
+  }
+
   librarymap_type& librarymap = getLibrarymap();
-  librarymap_type::iterator it = librarymap.find(libname);
+  librarymap_type::iterator it = librarymap.find(n);
   if (it == librarymap.end())
   {
     ComponentLibrary::factoryMapType factoryMap;
@@ -192,60 +265,45 @@ ComponentLibrary& Comploader::fetchLib(const std::string& libname)
     ValueResetter<ComponentLibrary::factoryMapType*> valueResetter(currentFactoryMap, 0);
 
     // load library
-    log_info("load library \"" << libname << '"');
+    log_info("load library \"" << n << '"');
     ComponentLibrary lib;
 
-    bool found = false;
     for (search_path_type::const_iterator p = search_path.begin();
-         p != search_path.end(); ++p)
+         !lib && p != search_path.end(); ++p)
     {
-      try
-      {
-        log_debug("load library \"" << libname << "\" from " << *p << " dir");
-        lib = ComponentLibrary(*p, libname);
-        found = true;
-        break;
-      }
-      catch (const cxxtools::dl::DlopenError&)
-      {
-      }
+      log_debug("load library \"" << n << "\" from " << *p << " dir");
+      lib = ComponentLibrary(*p, n, local);
+    }
+
+    if (!lib)
+    {
+      log_debug("load library \"" << n << "\" from current dir");
+      lib = ComponentLibrary(".", n, local);
     }
 
 #ifdef PKGLIBDIR
-    if (!found)
+    if (!lib)
     {
-      try
-      {
-        log_debug("load library \"" << libname << "\" from package lib dir");
-        lib = ComponentLibrary(PKGLIBDIR, libname);
-        found = true;
-      }
-      catch (const cxxtools::dl::DlopenError& e)
-      {
-      }
+      log_debug("load library \"" << n << "\" from package lib dir <" << PKGLIBDIR << '>');
+      lib = ComponentLibrary(PKGLIBDIR, n, local);
     }
 #endif
 
-    if (!found)
+    if (!lib)
     {
-      try
-      {
-        log_debug("load library \"" << libname << "\" from current dir");
-        lib = ComponentLibrary(".", libname);
-      }
-      catch (const cxxtools::dl::DlopenError& e)
-      {
-        log_debug("library \"" << e.getLibname() << "\" in current dir not found - search lib-path");
-        lib = ComponentLibrary(libname);
-      }
+      log_debug("library \"" << n << "\" in current dir not found - search lib-path");
+      lib = ComponentLibrary(n, local);
     }
 
+    if (!lib)
+      throw LibraryNotFound(n);
+
     lib.factoryMap = factoryMap;
-    log_debug("insert new library " << libname);
-    it = librarymap.insert(librarymap_type::value_type(libname, lib)).first;
+    log_debug("insert new library " << n);
+    it = librarymap.insert(librarymap_type::value_type(n, lib)).first;
   }
   else
-    log_debug("library " << libname << " found");
+    log_debug("library " << n << " found");
 
   return it->second;
 }
