@@ -1,20 +1,29 @@
-/* cookie.cpp
+/*
  * Copyright (C) 2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -33,6 +42,33 @@ namespace tnt
 {
   log_define("tntnet.cookie")
 
+  namespace
+  {
+    bool ishexdigit(char ch)
+    {
+      return ch >= '0' && ch <= '9'
+          || ch >= 'a' && ch <= 'f'
+          || ch >= 'A' && ch <= 'F';
+    }
+
+    char hexvalue(char ch)
+    {
+      if (ch >= '0' && ch <= '9')
+        return ch - '0';
+      else if (ch >= 'A' && ch <= 'F')
+        return ch - 'A' + 10;
+      else if (ch >= 'a' && ch <= 'f')
+        return ch - 'a' + 10;
+      else
+        return '\0';
+    }
+
+    char hexvalue(char upper, char lower)
+    {
+      return (hexvalue(upper) << 4) + hexvalue(lower);
+    }
+  }
+
   const Cookie Cookies::emptyCookie;
 
   const std::string Cookie::maxAge  = "Max-Age";
@@ -159,6 +195,14 @@ namespace tnt
     }
   }
 
+  namespace
+  {
+    void throwInvalidCookie(const std::string& cookie)
+    {
+      throw HttpError(HTTP_BAD_REQUEST, "invalid cookie: " + cookie);
+    }
+  }
+
   void CookieParser::parse(const std::string& header)
   {
     // Cookie: $Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"
@@ -170,12 +214,17 @@ namespace tnt
       state_eq,
       state_value0,
       state_value,
+      state_valueh1,
+      state_valueh2,
       state_valuee,
       state_qvalue,
+      state_qvalueh1,
+      state_qvalueh2,
       state_qvaluee
     };
 
     state_type state = state_0;
+    char h;
 
     for (std::string::const_iterator it = header.begin();
          it != header.end(); ++it)
@@ -202,7 +251,16 @@ namespace tnt
           if (std::isspace(ch))
             state = (name == Cookie::secure ? state_valuee : state_eq);
           else if (ch == '=')
-            state = (name == Cookie::secure ? state_valuee : state_value0);
+          {
+            if (name == Cookie::secure)
+              state = state_valuee;
+            else
+            {
+              value.clear();
+              value.reserve(32);
+              state = state_value0;
+            }
+          }
           else if (ch == ';' && name == Cookie::secure)
             state = state_valuee;
           else
@@ -211,25 +269,30 @@ namespace tnt
 
         case state_eq:
           if (ch == '=')
+          {
+            value.clear();
+            value.reserve(32);
             state = state_value0;
+          }
           else if (!std::isspace(ch))
           {
             log_warn("invalid cookie: " << header << " - '=' expected");
-            throw HttpError(HTTP_BAD_REQUEST, "invalid cookie: " + header);
+            throwInvalidCookie(header);
           }
           break;
 
         case state_value0:
           if (ch == '"')
-          {
-            value.clear();
-            value.reserve(32);
             state = state_qvalue;
+          else if (ch == '%')
+            state = state_valueh1;
+          else if (ch == ';')
+          {
+            process_nv();
+            state = state_0;
           }
           else if (!std::isspace(ch))
           {
-            value.clear();
-            value.reserve(32);
             value += ch;
             state = state_value;
           }
@@ -241,32 +304,146 @@ namespace tnt
             process_nv();
             state = state_0;
           }
+          else if (ch == '%')
+            state = state_valueh1;
+          else if (ch == '+')
+            value += ' ';
           else
             value += ch;
           break;
 
+        case state_valueh1:
+          if (ishexdigit(ch))
+          {
+            h = ch;
+            state = state_valueh2;
+          }
+          else if (ch == ';')
+          {
+            value += '%';
+            process_nv();
+            state = state_0;
+          }
+          else if (ch == '+')
+          {
+            value += '%';
+            value += ' ';
+            state = state_value;
+          }
+          else
+          {
+            value += '%';
+            value += ch;
+            state = state_value;
+          }
+          break;
+
+        case state_valueh2:
+          if (ishexdigit(ch))
+          {
+            value += hexvalue(h, ch);
+            state = state_value;
+          }
+          else if (ch == ';')
+          {
+            value += '%';
+            value += h;
+            process_nv();
+            state = state_0;
+          }
+          else if (ch == '+')
+          {
+            value += hexvalue(h);
+            value += ' ';
+            state = state_value;
+          }
+          else
+          {
+            value += hexvalue(h);
+            value += ch;
+            state = state_value;
+          }
+          break;
+
         case state_valuee:
           if (ch == ';')
           {
             process_nv();
             state = state_0;
           }
+          else if (ch == '+')
+            value += ' ';
           else if (std::isspace(ch))
             state = state_valuee;
           else
           {
             log_warn("invalid cookie: " << header << " - semicolon expected after value");
-            throw HttpError(HTTP_BAD_REQUEST, "invalid cookie: " + header);
+            throwInvalidCookie(header);
           }
           break;
 
         case state_qvalue:
           if (ch == '"')
             state = state_qvaluee;
+          else if (ch == '%')
+            state = state_qvalueh1;
+          else if (ch == '+')
+            value += ' ';
           else
             value += ch;
           break;
 
+        case state_qvalueh1:
+          if (ishexdigit(ch))
+          {
+            h = ch;
+            state = state_qvalueh2;
+          }
+          else if (ch == '"')
+          {
+            value += '%';
+            state = state_qvaluee;
+          }
+          else if (ch == '+')
+          {
+            value += '%';
+            value += ' ';
+            state = state_qvalue;
+          }
+          else
+          {
+            value += '%';
+            value += ch;
+            state = state_qvalue;
+          }
+          break;
+
+        case state_qvalueh2:
+          if (ishexdigit(ch))
+          {
+            value += hexvalue(h, ch);
+            state = state_qvalue;
+          }
+          else if (ch == '"')
+          {
+            value += '%';
+            value += h;
+            state = state_qvaluee;
+          }
+          else if (ch == '+')
+          {
+            value += hexvalue(h);
+            value += ' ';
+            state = state_qvalue;
+          }
+          else
+          {
+            value += hexvalue(h);
+            value += ch;
+            state = state_qvalue;
+          }
+          break;
+
         case state_qvaluee:
           if (ch == ';')
           {
@@ -276,24 +453,43 @@ namespace tnt
           else if (!std::isspace(ch))
           {
             log_warn("invalid cookie: " << header << " - semicolon expected");
-            throw HttpError(HTTP_BAD_REQUEST, "invalid cookie: " + header);
+            throwInvalidCookie(header);
           }
           break;
       }
     }
 
-    if (state == state_qvaluee || state == state_value)
+    if (state == state_qvaluee || state == state_value || state == state_value0)
       process_nv();
     else if (state != state_0)
     {
       log_warn("invalid cookie: " << header << " - invalid state " << state);
-      throw HttpError(HTTP_BAD_REQUEST, "invalid cookie: " + header);
+      throwInvalidCookie(header);
     }
 
     if (!current_cookie.value.empty())
       store_cookie();
   }
 
+  void Cookie::write(std::ostream& out, const std::string& name) const
+  {
+    // print name (Customer="WILE_E_COYOTE")
+    out << name << '=';
+    UrlEscOstream u(out);
+    u << getValue();
+
+    // print secure-attribute
+    if (secureFlag)
+      out << "; " << Cookie::secure;
+
+    // print attributes
+    for (Cookie::attrs_type::const_iterator a = attrs.begin();
+         a != attrs.end(); ++a)
+      out << "; " << a->first << '=' << a->second;
+    if (attrs.find(Cookie::version) == attrs.end())
+      out << ";Version=1";
+  }
+
   std::ostream& operator<< (std::ostream& out, const Cookies& c)
   {
     // Set-Cookie: Customer="WILE_E_COYOTE"; Version="1"; Path="/acme"
@@ -305,25 +501,10 @@ namespace tnt
       if (first)
         first = false;
       else
-        out << ' ';
+        out << ',';
 
-      const Cookie& cookie = it->second;
+      it->second.write(out, it->first);
 
-      // print name (Customer="WILE_E_COYOTE")
-      out << it->first << '=';
-      UrlEscOstream u(out);
-      u << cookie.getValue();
-
-      // print secure-attribute
-      if (cookie.secureFlag)
-        out << "; " << Cookie::secure;
-
-      // print attributes
-      for (Cookie::attrs_type::const_iterator a = cookie.attrs.begin();
-           a != cookie.attrs.end(); ++a)
-        out << "; " << a->first << '=' << a->second;
-      if (cookie.attrs.find(Cookie::version) == cookie.attrs.end())
-        out << "; Version=1";
     }
 
     return out;
