@@ -1,38 +1,53 @@
-/* encoding.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
 #include "tnt/encoding.h"
+#include "tnt/util.h"
 #include <cctype>
-#include <cxxtools/log.h>
-#include <stdexcept>
-
-log_define("tntnet.encoding")
 
 namespace tnt
 {
-  void Encoding::parse(const std::string& header)
+  namespace
   {
-    log_debug("encode header \"" << header << '"');
+    void throwInvalidHeader(const char* header)
+    {
+      throwRuntimeError(std::string("invalid encoding-string \"") + header + '"');
+    }
+  }
 
+  void Encoding::parse(const char* header)
+  {
     encodingMap.clear();
 
+    if (header == 0)
+      return;
+
     enum {
       state_0,
       state_encoding,
@@ -46,9 +61,9 @@ namespace tnt
 
     std::string encoding;
     unsigned quality;
-    for (std::string::const_iterator it = header.begin(); it != header.end(); ++it)
+    for (const char* p = header; *p; ++p)
     {
-      char ch = *it;
+      char ch = *p;
       switch (state)
       {
         case state_0:
@@ -66,7 +81,6 @@ namespace tnt
             state = state_qualityq;
           else if (ch == ',')
           {
-            log_debug("encoding=" << encoding);
             encodingMap.insert(encodingMapType::value_type(encoding, 1));
             state = state_0;
           }
@@ -78,14 +92,14 @@ namespace tnt
           if (ch == 'q')
             state = state_qualityeq;
           else if (!std::isspace(ch))
-            throw std::runtime_error("invalid encoding-string \"" + header + '"');
+            throwInvalidHeader(header);
           break;
 
         case state_qualityeq:
           if (ch == '=')
             state = state_quality;
           else if (!std::isspace(ch))
-            throw std::runtime_error("invalid encoding-string \"" + header + '"');
+            throwInvalidHeader(header);
           break;
 
         case state_quality:
@@ -95,7 +109,7 @@ namespace tnt
             state = state_qualitypoint;
           }
           else
-            throw std::runtime_error("invalid encoding-string \"" + header + '"');
+            throwInvalidHeader(header);
           break;
 
         case state_qualitypoint:
@@ -103,19 +117,17 @@ namespace tnt
             state = state_qualitytenth;
           else if (ch == ';')
           {
-            log_debug("encoding=" << encoding << " quality " << quality);
             encodingMap.insert(encodingMapType::value_type(encoding, quality));
             state = state_0;
           }
           else
-            throw std::runtime_error("invalid encoding-string \"" + header + '"');
+            throwInvalidHeader(header);
           break;
 
         case state_qualitytenth:
           if (std::isdigit(ch))
           {
             quality += ch - '0';
-            log_debug("encoding=" << encoding << " quality " << quality);
             encodingMap.insert(encodingMapType::value_type(encoding, quality));
             state = state_qualityign;
           }
@@ -133,14 +145,12 @@ namespace tnt
     switch (state)
     {
       case state_encoding:
-        log_debug("encoding=" << encoding);
         encodingMap.insert(encodingMapType::value_type(encoding, 1));
         break;
 
       case state_quality:
       case state_qualitypoint:
       case state_qualitytenth:
-        log_debug("encoding=" << encoding << " quality " << quality);
         encodingMap.insert(encodingMapType::value_type(encoding, quality));
         break;
 
@@ -153,25 +163,16 @@ namespace tnt
   {
     // check, if encoding is specified
 
-    log_debug("accept(\"" << encoding << "\")");
-
     encodingMapType::const_iterator it =  encodingMap.find(encoding);
     if (it != encodingMap.end())
-    {
-      log_debug("accept(\"" << encoding << "\") => " << it->second);
       return it->second;
-    }
 
     // check, if a wildcard-rule is specified
     it = encodingMap.find("*");
     if (it != encodingMap.end())
-    {
-      log_debug("accept(\"" << encoding << "\") => " << it->second);
       return it->second;
-    }
 
     // return 10 (accept), if encoding is identity, 0 otherwise
-    log_debug("accept(\"" << encoding << "\") => " << (encoding == "identity" ? 10 : 0));
     return encoding == "identity" ? 10 : 0;
   }
 
