@@ -1,35 +1,47 @@
-/* httprequest.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
 #include <tnt/httprequest.h>
 #include <tnt/httpparser.h>
+#include <tnt/httperror.h>
+#include <tnt/util.h>
 #include <sstream>
 #include <cxxtools/log.h>
-#include <cxxtools/thread.h>
+#include <cxxtools/mutex.h>
 #include <cxxtools/base64stream.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <tnt/sessionscope.h>
-#include <errno.h>
-#include <string.h>
+#include <tnt/socketif.h>
+#include <pthread.h>
 #include "config.h"
+#include <tnt/stringlessignorecase.h>
 
 namespace tnt
 {
@@ -39,31 +51,29 @@ namespace tnt
   // HttpRequest
   //
   size_t HttpRequest::maxRequestSize = 0;
-  unsigned HttpRequest::serial_ = 0;
+  cxxtools::atomic_t HttpRequest::serial_ = 0;
 
-  HttpRequest::HttpRequest(Tntnet& application_)
-    : ssl(false),
+  HttpRequest::HttpRequest(Tntnet& application_, const SocketIf* socketIf_)
+    : socketIf(socketIf_),
       locale_init(false),
       encodingRead(false),
       requestScope(0),
       applicationScope(0),
-      threadScope(0),
       sessionScope(0),
+      threadContext(0),
       applicationScopeLocked(false),
       sessionScopeLocked(false),
       application(application_)
   {
-    memset(&peerAddr, 0, sizeof(peerAddr));
-    memset(&serverAddr, 0, sizeof(peerAddr));
   }
 
-  HttpRequest::HttpRequest(Tntnet& application_, const std::string& url)
-    : ssl(false),
+  HttpRequest::HttpRequest(Tntnet& application_, const std::string& url, const SocketIf* socketIf_)
+    : socketIf(socketIf_),
       locale_init(false),
       requestScope(0),
       applicationScope(0),
-      threadScope(0),
       sessionScope(0),
+      threadContext(0),
       applicationScopeLocked(false),
       sessionScopeLocked(false),
       application(application_)
@@ -73,21 +83,20 @@ namespace tnt
   }
 
   HttpRequest::HttpRequest(const HttpRequest& r)
-    : pathinfo(r.pathinfo),
+    : methodLen(0),
+      pathinfo(r.pathinfo),
       args(r.args),
       qparam(r.qparam),
-      peerAddr(r.peerAddr),
-      serverAddr(r.serverAddr),
+      socketIf(r.socketIf),
       ct(r.ct),
       mp(r.mp),
-      ssl(r.ssl),
       serial(r.serial),
       locale_init(r.locale_init),
       locale(r.locale),
       requestScope(r.requestScope),
       applicationScope(r.applicationScope),
-      threadScope(r.threadScope),
       sessionScope(r.sessionScope),
+      threadContext(r.threadContext),
       applicationScopeLocked(false),
       sessionScopeLocked(false),
       application(r.application)
@@ -117,18 +126,16 @@ namespace tnt
     pathinfo = r.pathinfo;
     args = r.args;
     qparam = r.qparam;
-    peerAddr = r.peerAddr;
-    serverAddr = r.serverAddr;
     ct = r.ct;
     mp = r.mp;
-    ssl = r.ssl;
+    socketIf = r.socketIf;
     serial = r.serial;
     locale_init = r.locale_init;
     locale = r.locale;
     requestScope = r.requestScope;
     applicationScope = r.applicationScope;
-    threadScope = r.threadScope;
     sessionScope = r.sessionScope;
+    threadContext = r.threadContext;
     applicationScopeLocked = false;
     sessionScopeLocked = false;
 
@@ -146,7 +153,8 @@ namespace tnt
   {
     HttpMessage::clear();
     body.clear();
-    method.clear();
+    methodLen = 0;
+    method[0] = '\0';
     url.clear();
     queryString.clear();
     contentSize = 0;
@@ -174,13 +182,20 @@ namespace tnt
       applicationScope = 0;
     }
 
-    threadScope = 0;
-
     if (sessionScope)
     {
       sessionScope->release();
       sessionScope = 0;
     }
+
+    threadContext = 0;
+  }
+
+  void HttpRequest::setMethod(const char* m)
+  {
+    if (strlen(m) >= 7)
+      throw HttpError(HTTP_BAD_REQUEST, "invalid method");
+    strcpy(method, m);
   }
 
   std::string HttpRequest::getArg(const std::string& name, const std::string& def) const
@@ -204,18 +219,15 @@ namespace tnt
   void HttpRequest::doPostParse()
   {
     qparam.parse_url(getQueryString());
-    if (getMethod() == "POST")
+    if (isMethodPOST())
     {
       std::istringstream in(getHeader(httpheader::contentType));
       in >> ct;
 
       if (in)
       {
-        log_debug(httpheader::contentType << ' ' << in.str());
-        log_debug("type=" << ct.getType() << " subtype=" << ct.getSubtype());
         if (ct.isMultipart())
         {
-          log_debug("multipart-boundary=" << ct.getBoundary());
           mp.set(ct.getBoundary(), getBody());
           for (Multipart::const_iterator it = mp.begin();
                it != mp.end(); ++it)
@@ -225,60 +237,19 @@ namespace tnt
             if (it->getFilename().empty())
             {
               std::string multipartBody(it->getBodyBegin(), it->getBodyEnd());
-              log_debug("multipart-item name=" << it->getName()
-                     << " body=" << multipartBody);
               qparam.add(it->getName(), multipartBody);
             }
           }
         }
-        else
+        else if (ct.getType() == "application"
+              && ct.getSubtype() == "x-www-form-urlencoded")
         {
           qparam.parse_url(getBody());
         }
       }
-      else if (ct.getType() == "application"
-            && ct.getSubtype() == "x-www-form-urlencoded")
-        qparam.parse_url(getBody());
     }
 
-    {
-      static cxxtools::Mutex monitor;
-      cxxtools::MutexLock lock(monitor);
-      serial = ++serial_;
-    }
-  }
-
-  namespace
-  {
-    void formatIp(const sockaddr_storage& addr, std::string& str)
-    {
-#ifdef HAVE_INET_NTOP
-      const sockaddr_in* sa = reinterpret_cast<const sockaddr_in*>(&addr);
-      char strbuf[INET6_ADDRSTRLEN + 1];
-      const char* p = inet_ntop(sa->sin_family, &sa->sin_addr, strbuf, sizeof(strbuf));
-      str = (p == 0 ? "-" : strbuf);
-#else
-      static cxxtools::Mutex monitor;
-      cxxtools::MutexLock lock(monitor);
-
-      const char* p = inet_ntoa(peerAddr.sin_addr);
-      str = (p == 0 ? "-" : strbuf);
-#endif
-    }
-  }
-
-  std::string HttpRequest::getPeerIp() const
-  {
-    if (peerAddrStr.empty())
-      formatIp(peerAddr, peerAddrStr);
-    return peerAddrStr;
-  }
-
-  std::string HttpRequest::getServerIp() const
-  {
-    if (serverAddrStr.empty())
-      formatIp(serverAddr, serverAddrStr);
-    return serverAddrStr;
+    serial = cxxtools::atomicIncrement(serial_);
   }
 
   namespace
@@ -364,16 +335,11 @@ namespace tnt
 
   const Cookies& HttpRequest::getCookies() const
   {
-    log_debug("HttpRequest::getCookies()");
-
     if (!httpcookies.hasCookies())
     {
       header_type::const_iterator it = header.find(httpheader::cookie);
       if (it != header.end())
-      {
-        log_debug("parse cookie-header " << it->second);
         const_cast<HttpRequest*>(this)->httpcookies.set(it->second);
-      }
     }
 
     return httpcookies;
@@ -399,9 +365,8 @@ namespace tnt
       cxxtools::Base64istream in(authHeader);
       std::getline(in, username, ':');
       std::getline(in, password);
-
-      log_debug("username \"" << username << "\" password \"" << password << '"');
     }
+
     return username;
   }
 
@@ -414,29 +379,19 @@ namespace tnt
   bool HttpRequest::verifyPassword(const std::string& password_) const
   {
     getUsername();
-    log_debug("verify password \"" << password_ << "\" for username \"" << username << "\" password \"" << password << '"');
     return password == password_;
   }
 
-  namespace
-  {
-    class CompareNoCase
-    {
-      public:
-        bool operator() (char c1, char c2) const
-          { return tolower(c1) == tolower(c2); }
-    };
-  }
-
   bool HttpRequest::keepAlive() const
   {
-    header_type::const_iterator it = header.find(httpheader::connection);
-    return it == header.end() ? getMajorVersion() == 1 && getMinorVersion() == 1
-                              : it->second.size() >= httpheader::connectionKeepAlive.size()
-                                && std::equal(it->second.begin(),
-                                              it->second.end(),
-                                              httpheader::connectionKeepAlive.begin(),
-                                              CompareNoCase());
+    // request is keep alive when either a connection header is explicitely set to keep alive
+    // or the http version is 1.1
+    Messageheader::const_iterator it = header.find(httpheader::connection);
+
+    return it == header.end()
+               ? getMinorVersion() >= 1 && getMajorVersion() >= 1
+               : tnt::StringCompareIgnoreCase<const char*>(it->second,
+                    httpheader::connectionKeepAlive) == 0;
   }
 
   const Contenttype& HttpRequest::getContentTypePriv() const
@@ -462,11 +417,6 @@ namespace tnt
     applicationScope = s;
   }
 
-  void HttpRequest::setThreadScope(Scope* s)
-  {
-    threadScope = s;
-  }
-
   void HttpRequest::setSessionScope(Sessionscope* s)
   {
     if (sessionScope == s)
@@ -496,61 +446,41 @@ namespace tnt
 
   void HttpRequest::ensureApplicationScopeLock()
   {
-    log_trace("ensureApplicationScopeLock; thread " << pthread_self());
-
     ensureSessionScopeLock();
     if (applicationScope && !applicationScopeLocked)
     {
-      log_debug("lock application scope; thread" << pthread_self());
       applicationScope->lock();
       applicationScopeLocked = true;
     }
-    else
-      log_debug("applicationscope locked already");
   }
 
   void HttpRequest::ensureSessionScopeLock()
   {
-    log_trace("ensureSessionScopeLock; thread " << pthread_self());
-
     if (sessionScope && !sessionScopeLocked)
     {
-      log_debug("lock sessionscope; thread " << pthread_self());
       sessionScope->lock();
       sessionScopeLocked = true;
     }
-    else
-      log_debug("sessionscope locked already");
   }
 
   void HttpRequest::releaseApplicationScopeLock()
   {
-    log_trace("releaseApplicationScopeLock; thread " << pthread_self());
+    releaseSessionScopeLock();
 
     if (applicationScope && applicationScopeLocked)
     {
-      log_debug("unlock applicationscope");
       applicationScopeLocked = false;
       applicationScope->unlock();
     }
-    else
-      log_debug("applicationscope not locked");
   }
 
   void HttpRequest::releaseSessionScopeLock()
   {
-    log_trace("releaseSessionScopeLock; thread " << pthread_self());
-
-    releaseApplicationScopeLock();
-
     if (sessionScope && sessionScopeLocked)
     {
-      log_debug("unlock sessionscope");
       sessionScopeLocked = false;
       sessionScope->unlock();
     }
-    else
-      log_debug("sessionscope not locked");
   }
 
   Scope& HttpRequest::getRequestScope()
@@ -562,9 +492,9 @@ namespace tnt
 
   Scope& HttpRequest::getThreadScope()
   {
-    if (threadScope == 0)
-      throw std::runtime_error("threadscope not set");
-    return *threadScope;
+    if (threadContext == 0)
+      throwRuntimeError("threadcontext not set");
+    return threadContext->getScope();
   }
 
   Scope& HttpRequest::getApplicationScope()
