@@ -1,27 +1,36 @@
-/* tnt/httpparser.h
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
 #ifndef TNT_HTTPPARSER_H
 #define TNT_HTTPPARSER_H
 
-#include <tnt/httpmessage.h>
+#include <tnt/httprequest.h>
 #include <tnt/http.h>
 #include <tnt/parser.h>
 #include <tnt/messageheaderparser.h>
@@ -46,10 +55,10 @@ namespace tnt
       void reset()  { requestSize = 0; }
   };
 
-  class HttpMessage::Parser
-    : public tnt::Parser<HttpMessage::Parser, RequestSizeMonitor>
+  class HttpRequest::Parser
+    : public tnt::Parser<HttpRequest::Parser, RequestSizeMonitor>
   {
-      HttpMessage& message;
+      HttpRequest& message;
       Messageheader::Parser headerParser;
 
       unsigned httpCode;
@@ -75,7 +84,7 @@ namespace tnt
       virtual void requestSizeExceeded();
 
     public:
-      Parser(tnt::HttpMessage& message_)
+      Parser(tnt::HttpRequest& message_)
         : tnt::Parser<Parser, RequestSizeMonitor>(&Parser::state_cmd0),
           message(message_),
           headerParser(message_.header),
