@@ -1,20 +1,29 @@
-/* tnt/httpreply.h
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -53,19 +62,24 @@ namespace tnt
       bool sendStatusLine;
       bool headRequest;
 
-      void tryCompress(std::string& body);
-      void send(unsigned ret, const char* msg, bool ready);
+      void send(unsigned ret, const char* msg, bool ready) const;
 
     public:
       explicit HttpReply(std::ostream& s, bool sendStatusLine = true);
 
+      static bool tryCompress(std::string& body);
+
+      void setContentType(const char* t)            { setHeader(httpheader::contentType, t); }
       void setContentType(const std::string& t)     { setHeader(httpheader::contentType, t); }
-      std::string getContentType() const            { return getHeader(httpheader::contentType); }
+      const char* getContentType() const            { return getHeader(httpheader::contentType); }
 
       void setHeadRequest(bool sw = true)           { headRequest = sw; }
 
+      /// Throws an exception, which results in a redirect.
       unsigned redirect(const std::string& newLocation);
+      /// Throws an exception, which results in a login dialog in the browser.
       unsigned notAuthorized(const std::string& realm);
+      /// alias for notAuthorized
       unsigned notAuthorised(const std::string& realm) { return notAuthorized(realm); }
 
       void sendReply(unsigned ret, const char* msg = "OK");
@@ -119,7 +133,6 @@ namespace tnt
       static const std::string& getDefaultContentType()        { return defaultContentType; }
 
       void setAcceptEncoding(const Encoding& enc)    { acceptEncoding = enc; }
-      void setAcceptEncoding(const std::string& enc) { acceptEncoding.parse(enc); }
 
       bool keepAlive() const;
 
