@@ -1,20 +1,29 @@
 /*
  * Copyright (C) 2006 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -165,7 +174,7 @@ namespace tnt
       theProcess = 0;
   }
 
-  void Process::runMonitor(cxxtools::Pipe& mainPipe)
+  void Process::runMonitor(cxxtools::posix::Pipe& mainPipe)
   {
     log_debug("run monitor");
 
@@ -177,7 +186,7 @@ namespace tnt
 
     while (true)
     {
-      cxxtools::Pipe monitorPipe;
+      cxxtools::posix::Pipe monitorPipe;
 
       cxxtools::Fork fork;
 
@@ -287,6 +296,7 @@ namespace tnt
     }
 
     signal(SIGTERM, sigEnd);
+    signal(SIGINT, sigEnd);
     signal(SIGHUP, sigReload);
   }
 
@@ -298,7 +308,7 @@ namespace tnt
 
       // We receive the writing-end of the notify pipe.
       // After successful initialization we need to write a byte to this fd.
-      cxxtools::Pipe mainPipe;
+      cxxtools::posix::Pipe mainPipe;
       cxxtools::Fork fork;
       if (fork.parent())
       {
