@@ -1,20 +1,29 @@
-/* parser.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -63,26 +72,18 @@ namespace tnt
         throw std::runtime_error("cannot open include file \"" + file + '"');
       }
 
-      try
-      {
-        std::string curfileSave = curfile;
-        unsigned curlineSave = curline;
-        curfile = fullname;
-        curline = 0;
+      std::string curfileSave = curfile;
+      unsigned curlineSave = curline;
+      curfile = fullname;
+      curline = 0;
 
-        log_debug("onInclude(\"" << fullname << "\")");
-        handler.onInclude(fullname);
+      log_debug("onInclude(\"" << fullname << "\")");
+      handler.onInclude(fullname);
 
-        parsePriv(inp);
+      parsePriv(inp);
 
-        curfile = curfileSave;
-        curline = curlineSave;
-      }
-      catch (const std::exception&)
-      {
-        std::cerr << "file " << file << std::endl;
-        throw;
-      }
+      curfile = curfileSave;
+      curline = curlineSave;
 
       log_debug("onIncludeEnd(\"" << fullname << "\")");
       handler.onIncludeEnd(fullname);
@@ -540,6 +541,8 @@ namespace tnt
               else
                 state = state_cpp;
             }
+            else if (tag == "def" && ch == '(')
+              state = state_defarg0;
             break;
 
           case state_defarg0:
@@ -607,7 +610,7 @@ namespace tnt
               state = state_html0;
             }
             else if (!std::isspace(ch))
-              throw parse_error("def", state, curline);
+              throw parse_error("def", state, curfile, curline);
             break;
 
           case state_cppcomment0:
@@ -743,7 +746,7 @@ namespace tnt
               if (tag != etag)
                 throw parse_error("invalid end-tag - "
                   + tag + " expected, "
-                  + etag + " found", state, curline);
+                  + etag + " found", state, curfile, curline);
               else if (tag == "pre")
               {
                 log_debug("onPre(\"" << code << "\")");
@@ -769,6 +772,14 @@ namespace tnt
                 log_debug("onCpp(\"" << code << "\")");
                 handler.onCpp(code);
               }
+              else if (tag == "out")
+              {
+                handler.onHtmlExpression(code);
+              }
+              else if (tag == "sout")
+              {
+                handler.onExpression(code);
+              }
               else if (tag == "args" || tag == "attr" || tag == "config"
                 || tag == "include"
                 || tag == "session"
@@ -779,7 +790,7 @@ namespace tnt
                 || tag == "doc")
                 ;
               else
-                throw parse_error("unknown tag " + tag, state, curline);
+                throw parse_error("unknown tag " + tag, state, curfile, curline);
               code.clear();
               state = state_html0;
             }
@@ -858,7 +869,7 @@ namespace tnt
             if (ch == '/')
               state = state_argscomment;
             else
-              throw parse_error("7", state, curline);
+              throw parse_error("7", state, curfile, curline);
             break;
 
           case state_argsvar:
@@ -895,7 +906,7 @@ namespace tnt
               arg.clear();
               state = state_args0;
               if (ch == '\n')
-                std::cerr << "old syntax: ';' missing in line " << curline << std::endl;
+                std::cerr << curfile << ':' << curline << ": warning: old syntax: ';' missing" << std::endl;
             }
             else if (!std::isspace(ch))
             {
@@ -922,7 +933,7 @@ namespace tnt
               value.clear();
               state = state_args0;
               if (ch == '\n')
-                std::cerr << "old syntax: ';' missing in line " << curline << std::endl;
+                std::cerr << curfile << ':' << curline << ": warning: old syntax: ';' missing" << std::endl;
             }
             else if (ch == '"')
             {
@@ -945,7 +956,7 @@ namespace tnt
             if (ch == '/')
               state = state_argscomment;
             else
-              throw parse_error("6", state, curline);
+              throw parse_error("6", state, curfile, curline);
             break;
 
           case state_argscomment:
@@ -961,7 +972,7 @@ namespace tnt
               value.clear();
               state = state_argscomment;
               if (ch == '\n')
-                std::cerr << "old syntax: ';' missing in line " << curline << std::endl;
+                std::cerr << curfile << ':' << curline << ": warning: old syntax: ';' missing" << std::endl;
             }
             else
             {
@@ -987,7 +998,7 @@ namespace tnt
             if (ch == '/')
               state = state_attrcomment;
             else
-              throw parse_error("7", state, curline);
+              throw parse_error("7", state, curfile, curline);
             break;
 
           case state_attrvar:
@@ -1006,26 +1017,26 @@ namespace tnt
             if (ch == '=')
             {
               if (arg.empty())
-                throw parse_error("name expected", state, curline);
+                throw parse_error("name expected", state, curfile, curline);
               value.clear();
               state = state_attrval;
             }
             else if (!std::isspace(ch))
-              throw parse_error("'=' expected", state, curline);
+              throw parse_error("'=' expected", state, curfile, curline);
             break;
 
           case state_attrval:
             if (ch == '\n' || ch == ';')
             {
               if (value.empty())
-                throw parse_error("value expected", state, curline);
+                throw parse_error("value expected", state, curfile, curline);
               log_debug("onAttr(\"" << arg << "\", \"" << value << "\")");
               handler.onAttr(arg, value);
               arg.clear();
               value.clear();
               state = state_attr0;
               if (ch == '\n')
-                std::cerr << "old syntax: ';' missing in line " << curline << std::endl;
+                std::cerr << curfile << ':' << curline << ": warning: old syntax: ';' missing" << std::endl;
             }
             else if (ch == '"')
             {
@@ -1048,7 +1059,7 @@ namespace tnt
             if (ch == '/')
               state = state_attrcomment;
             else
-              throw parse_error("6", state, curline);
+              throw parse_error("6", state, curfile, curline);
             break;
 
           case state_attrcomment:
@@ -1060,7 +1071,7 @@ namespace tnt
             if (ch == '/')
             {
               if (value.empty())
-                throw parse_error("value expected", state, curline);
+                throw parse_error("value expected", state, curfile, curline);
               log_debug("onAttr(\"" << arg << "\", \"" << value << "\")");
               handler.onAttr(arg, value);
               arg.clear();
@@ -1107,7 +1118,7 @@ namespace tnt
             if (ch == '"')
               state = state_callarg0;
             else if (std::isspace(ch))
-              throw parse_error("invalid componentname", state, curline);
+              throw parse_error("invalid componentname", state, curfile, curline);
             break;
 
           case state_callname:
@@ -1153,7 +1164,7 @@ namespace tnt
 
           case state_call_cpparg1:
             if (ch == ')')
-              throw parse_error("')' unexpected", state, curline);
+              throw parse_error("')' unexpected", state, curfile, curline);
             else if (std::isalpha(ch) || ch == '_')
             {
               expr = ch;
@@ -1238,7 +1249,7 @@ namespace tnt
               else
               {
                 if (paramargs.find(paramname) != paramargs.end())
-                  throw parse_error("duplicate parameter " + paramname, state, curline);
+                  throw parse_error("duplicate parameter " + paramname, state, curfile, curline);
                 paramargs[paramname] = expr;
                 paramname.clear();
                 expr.clear();
@@ -1257,7 +1268,7 @@ namespace tnt
             if (ch == '>')
               state = state_html;
             else
-              throw parse_error("3", state, curline);
+              throw parse_error("3", state, curfile, curline);
             break;
 
           case state_callarg0:
@@ -1281,7 +1292,7 @@ namespace tnt
             else if (ch == '(' && arg.empty() && pass_cgi.empty())
               state = state_call_cpparg0;
             else if (!std::isspace(ch))
-              throw parse_error(std::string("invalid argumentname (") + ch + ')', state, curline);
+              throw parse_error(std::string("invalid argumentname (") + ch + ')', state, curfile, curline);
             break;
 
           case state_callarg:
@@ -1303,7 +1314,7 @@ namespace tnt
               state = ch == '>' ? state_html : state_callend;
             }
             else
-              throw parse_error(std::string("invalid argumentname (") + ch + ')', state, curline);
+              throw parse_error(std::string("invalid argumentname (") + ch + ')', state, curfile, curline);
             break;
 
           case state_callarge:
@@ -1327,7 +1338,7 @@ namespace tnt
               state = ch == '>' ? state_html : state_callend;
             }
             else if (!std::isspace(ch))
-              throw parse_error("5", state, curline);
+              throw parse_error("5", state, curfile, curline);
             break;
 
           case state_callval0:
@@ -1402,7 +1413,7 @@ namespace tnt
               state = state_html;
             }
             else
-              throw parse_error("ivalid value", state, curline);
+              throw parse_error("ivalid value", state, curfile, curline);
             break;
 
           case state_comment:
@@ -1559,14 +1570,14 @@ namespace tnt
             {
               state = state_scopeargval0;
               if (tagarg != "scope")
-                throw parse_error("argument \"scope\" expected", state, curline);
+                throw parse_error("argument \"scope\" expected", state, curfile, curline);
               tagarg.clear();
             }
             else if (std::isspace(ch))
             {
               state = state_scopeargeq;
               if (tagarg != "scope")
-                throw parse_error("argument \"scope\" expected", state, curline);
+                throw parse_error("argument \"scope\" expected", state, curfile, curline);
               tagarg.clear();
             }
             else
@@ -1577,14 +1588,14 @@ namespace tnt
             if (ch == '=')
               state = state_scopeargval0;
             else if (!std::isspace(ch))
-              throw parse_error("\"=\" expected", state, curline);
+              throw parse_error("\"=\" expected", state, curfile, curline);
             break;
 
           case state_scopeargval0:
             if (ch == '"')
               state = state_scopeargval;
             else if (!std::isspace(ch))
-              throw parse_error("argument expected", state, curline);
+              throw parse_error("argument expected", state, curfile, curline);
             break;
 
           case state_scopeargval:
@@ -1597,13 +1608,13 @@ namespace tnt
               else if (value == "component")
                 scope = component_scope;
               else
-                throw parse_error("global|page|component expected", state, curline);
+                throw parse_error("global|page|component expected", state, curfile, curline);
 
               value.clear();
               state = state_scopevale;
             }
             else if (ch == '\n')
-              throw parse_error("'\"' expected", state, curline);
+              throw parse_error("'\"' expected", state, curfile, curline);
             else
               value += ch;
             break;
@@ -1612,7 +1623,7 @@ namespace tnt
             if (ch == '>')
               state = state_scope0;
             else if (!std::isspace(ch))
-              throw parse_error("'>' expected", state, curline);
+              throw parse_error("'>' expected", state, curfile, curline);
             break;
 
           case state_scope0:
@@ -1708,7 +1719,7 @@ namespace tnt
               else if (ch == ')')
               {
                 if (bracket_count == 0)
-                  throw parse_error("unexpected ')'", state, curline);
+                  throw parse_error("unexpected ')'", state, curfile, curline);
                 --bracket_count;
               }
             }
@@ -1742,7 +1753,7 @@ namespace tnt
               else if (ch == ')')
               {
                 if (bracket_count == 0)
-                  throw parse_error("unexpected ')'", state, curline);
+                  throw parse_error("unexpected ')'", state, curfile, curline);
                 --bracket_count;
               }
             }
@@ -1752,14 +1763,14 @@ namespace tnt
             if (ch == ';')
               state = state_scope0;
             else if (!std::isspace(ch))
-              throw parse_error("invalid scopedefinition", state, curline);
+              throw parse_error("invalid scopedefinition", state, curfile, curline);
             break;
 
           case state_scopecomment0:
             if (ch == '/')
               state = state_scopecomment;
             else
-              throw parse_error("invalid scopedefinition - '/' expexted", state, curline);
+              throw parse_error("invalid scopedefinition - '/' expexted", state, curfile, curline);
             break;
 
           case state_scopecomment:
@@ -1788,7 +1799,7 @@ namespace tnt
             }
             else if (!std::isspace(ch))
               throw parse_error(std::string("character expected, \'") + ch
-                + "\' found", state, curline);
+                + "\' found", state, curfile, curline);
             break;
 
           case state_endcall:
@@ -1815,14 +1826,14 @@ namespace tnt
             else if (isVariableNameChar(ch) || ch == '@')
               comp += ch;
             else
-              throw parse_error("character expected", state, curline);
+              throw parse_error("character expected", state, curfile, curline);
             break;
 
           case state_endcalle:
             if (ch == '>')
               state = state_html;
             else if (!std::isspace(ch))
-              throw parse_error("'>' expected", state, curline);
+              throw parse_error("'>' expected", state, curfile, curline);
             break;
 
           case state_doc:
@@ -1853,13 +1864,13 @@ namespace tnt
       }  // while(in.get(ch))
 
       if (state != state_html && state != state_html0 && state != state_nl)
-        throw parse_error("parse error", state, curline);
+        throw parse_error("parse error", state, curfile, curline);
 
       if (inComp)
-        throw parse_error("</%def> missing", state, curline);
+        throw parse_error("</%def> missing", state, curfile, curline);
 
       if (inClose)
-        throw parse_error("</%close> missing", state, curline);
+        throw parse_error("</%close> missing", state, curfile, curline);
 
       if (!html.empty())
       {
@@ -1883,11 +1894,11 @@ namespace tnt
         handler.onConfig(name, value);
     }
 
-    parse_error::parse_error(const std::string& txt, int state, unsigned curline)
+    parse_error::parse_error(const std::string& txt, int state, const std::string& file, unsigned curline)
       : runtime_error(std::string())
     {
       std::ostringstream m;
-      m << "ERROR: " << txt << " in state " << state << " at line " << curline;
+      m << file << ':' << curline << ": error: " << txt << " (state " << state << ')';
       msg = m.str();
     }
 
