@@ -1,20 +1,29 @@
-/* ecppc.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -42,36 +51,47 @@ namespace tnt
       : requestname(cxxtools::Arg<std::string>(argc, argv, 'n')),
         ofile(cxxtools::Arg<std::string>(argc, argv, 'o')),
         odir(cxxtools::Arg<std::string>(argc, argv, 'O')),
-        mimetype(argc, argv, 'm'),
-        mimedb(argc, argv, "--mimetypes", "/etc/mime.types"),
-        binary(argc, argv, 'b'),
-        multibinary(argc, argv, 'b'),
-        componentclass(argc, argv, 'C'),
-        compress(argc, argv, 'z'),
-        verbose(argc, argv, 'v'),
-        debug(argc, argv, 'd'),
-        trace(argc, argv, 't'),
-        generateDependencies(argc, argv, 'M'),
-        generateHeader(argc, argv, 'h'),
-        disableLinenumbers(argc, argv, 'L')
+        mimetype(cxxtools::Arg<std::string>(argc, argv, 'm')),
+        mimedb(cxxtools::Arg<std::string>(argc, argv, "--mimetypes", "/etc/mime.types"))
     {
+
       while (true)
       {
-        cxxtools::Arg<const char*> include(argc, argv, 'I', 0);
+        cxxtools::Arg<std::string> include(argc, argv, 'I');
         if (!include.isSet())
           break;
-        includes.push_back(include.getValue());
+        log_debug("include: " << include.getValue());
+        includes.push_back(include);
       }
 
+      binary = cxxtools::Arg<bool>(argc, argv, 'b');
+      multibinary = cxxtools::Arg<bool>(argc, argv, 'b');
+      keepPath = cxxtools::Arg<bool>(argc, argv, 'p');
+      compress = cxxtools::Arg<bool>(argc, argv, 'z');
+      verbose = cxxtools::Arg<bool>(argc, argv, 'v');
+      generateDependencies = cxxtools::Arg<bool>(argc, argv, 'M');
+      disableLinenumbers = cxxtools::Arg<bool>(argc, argv, 'L');
+
       if (argc < 2 || argv[1][0] == '-')
         throw Usage(argv[0]);
       inputfile = argv[1];
-      std::copy(argv + 1, argv + argc, std::inserter(inputfiles, inputfiles.end()));
+      if (multibinary)
+      {
+        for (int a = 1; a < argc; ++a)
+        {
+          std::string inputfile = argv[a];
+          std::string key = inputfile;
+          if (!keepPath)
+          {
+            // strip path
+            std::string::size_type p;
+            if ((p = key.find_last_of('/')) != std::string::npos)
+              key.erase(0, p + 1);
+          }
 
-      if (debug)
-        log_init_debug();
-      else
-        log_init();
+          inputfiles.insert(inputfiles_type::value_type(key, inputfile));
+        }
+      }
     }
 
     int Ecppc::run()
@@ -83,18 +103,15 @@ namespace tnt
         {
           std::cerr << "warning: no requestname passed (with -n) - using \"images\"" << std::endl;
           requestname = "images";
-
-          if (ofile.empty() && !generateDependencies)
-            ofile = requestname;
         }
         else
         {
           std::string input = inputfile;
           std::string::size_type pos_dot = input.find_last_of(".");
-          std::string::size_type pos_slash = input.find_last_of("\\/");
           if (pos_dot != std::string::npos)
           {
-            if (pos_slash == std::string::npos)
+            std::string::size_type pos_slash;
+            if (keepPath || (pos_slash = input.find_last_of("\\/")) == std::string::npos)
               requestname = input.substr(0, pos_dot);
             else if (pos_slash < pos_dot)
               requestname = input.substr(pos_slash + 1, pos_dot - pos_slash - 1);
@@ -111,12 +128,15 @@ namespace tnt
 
           if (requestname.empty())
           {
-            std::cerr << "cannot derive classname from filename. Use -n" << std::endl;
+            std::cerr << "cannot derive component name from filename. Use -n" << std::endl;
             return -1;
           }
         }
       }
 
+      if (ofile.empty() && !generateDependencies)
+        ofile = requestname;
+
       if (generateDependencies)
         return runDependencies();
       else
@@ -133,11 +153,10 @@ namespace tnt
       tnt::ecppc::Generator generator(requestname);
 
       // initialize
-      generator.setDebug(trace);
       generator.enableLinenumbers(!disableLinenumbers);
       Bodypart::enableLinenumbers(!disableLinenumbers);
 
-      if (mimetype.isSet())
+      if (!mimetype.empty())
         generator.setMimetype(mimetype);
       else if (!extname.empty() && !multibinary)
       {
@@ -154,9 +173,6 @@ namespace tnt
 
       generator.setCompress(compress);
 
-      if (componentclass.isSet())
-        generator.setComponentclass(componentclass);
-
       std::string obase = odir;
       if (!obase.empty())
         obase += '/';
@@ -169,18 +185,35 @@ namespace tnt
       if (multibinary)
       {
         tnt::MimeDb mimeDb;
-        if (!mimetype.isSet())
+        if (mimetype.empty())
           mimeDb.read(mimedb);
 
         for (inputfiles_type::const_iterator it = inputfiles.begin();
              it != inputfiles.end(); ++it)
         {
-          std::string inputfile = *it;
+          std::string key = it->first;
+          std::string inputfile = it->second;
 
           struct stat st;
+          log_debug("check for input file " << inputfile);
           if (stat(inputfile.c_str(), &st) != 0)
-            throw std::runtime_error("can't stat " + inputfile);
+          {
+            // search for input file in includes list
+            int ret;
+            for (includes_type::const_iterator incl = includes.begin();
+              incl != includes.end(); ++incl)
+            {
+              std::string inputfile_ = *incl + '/' + it->second;
+              log_debug("check for input file " << inputfile_);
+              if ((ret = stat(inputfile_.c_str(), &st)) == 0)
+              {
+                inputfile = inputfile_;
+                break;
+              }
+            }
+          }
 
+          log_debug("read input file " << inputfile);
           std::ifstream in(inputfile.c_str());
           if (!in)
             throw std::runtime_error("can't read " + inputfile);
@@ -189,21 +222,19 @@ namespace tnt
           content << in.rdbuf();
 
           std::string mime;
-          if (mimetype.isSet())
+          if (!mimetype.empty())
             mime = mimetype;
           else
           {
             mime = mimeDb.getMimetype(inputfile);
             if (mime.empty())
-              std::cerr << "warning: no mimetype found for \"" << inputfile << '"' << std::endl;
+            {
+              mime = "application/x-data";
+              std::cerr << "warning: no mimetype found for \"" << inputfile << "\" using " << mime << std::endl;
+            }
           }
 
-          // strip path
-          std::string::size_type p;
-          if ((p = inputfile.find_last_of('/')) != std::string::npos)
-            inputfile.erase(0, p + 1);
-
-          generator.addImage(inputfile, content.str(), mime, st.st_ctime);
+          generator.addImage(key, content.str(), mime, st.st_ctime);
         }
       }
       else
@@ -230,37 +261,14 @@ namespace tnt
       //
       // generate Code
       //
-      if (generateHeader)
-      {
-        if (verbose)
-          std::cout << "generate " << obase << ".h" << std::endl;
-        std::ofstream hout((obase + ".h").c_str());
-        generator.getHeader(hout, ofile + ".h");
-        hout.close();
+      if (verbose)
+        std::cout << "generate " << obase << ".cpp" << std::endl;
+      std::ofstream sout((obase + ".cpp").c_str());
+      generator.getCpp(sout, ofile + ".cpp");
+      sout.close();
 
-        if (!hout)
-          throw std::runtime_error("error writing file \"" + ofile + ".h\"");
-
-        if (verbose)
-          std::cout << "generate " << obase << ".cpp" << std::endl;
-        std::ofstream sout((obase + ".cpp").c_str());
-        generator.getCpp(sout, ofile + ".cpp");
-        sout.close();
-
-        if (!sout)
-          throw std::runtime_error("error writing file \"" + ofile + ".cpp\"");
-      }
-      else
-      {
-        if (verbose)
-          std::cout << "generate " << obase << ".cpp" << std::endl;
-        std::ofstream sout((obase + ".cpp").c_str());
-        generator.getCppWoHeader(sout, ofile + ".cpp");
-        sout.close();
-
-        if (!sout)
-          throw std::runtime_error("error writing file \"" + ofile + ".cpp\"");
-      }
+      if (!sout)
+        throw std::runtime_error("error writing file \"" + ofile + ".cpp\"");
 
       return 0;
     }
@@ -279,11 +287,11 @@ namespace tnt
         runParser(in, generator);
 
       if (ofile.empty())
-        generator.getDependencies(std::cout, generateHeader);
+        generator.getDependencies(std::cout);
       else
       {
         std::ofstream out(ofile.c_str());
-        generator.getDependencies(out, generateHeader);
+        generator.getDependencies(out);
       }
 
       return 0;
@@ -310,7 +318,7 @@ namespace tnt
            "ecppc-compiler\n\n"
            "usage: " << progname << " [options] ecpp-source\n\n"
            "  -o filename      outputfile\n"
-           "  -n name          classname\n"
+           "  -n name          componentname\n"
            "  -I dir           include-directory\n"
            "  -m type          Mimetype\n"
            "  --mimetypes file read mimetypes from file (default /etc/mime.types)\n"
@@ -318,9 +326,8 @@ namespace tnt
            "  -bb              generate multibinary component\n"
            "  -z               compress constant data\n"
            "  -v               verbose\n"
-           "  -t               generate traces\n"
            "  -M               generate dependency for Makefile\n"
-           "  -h               generate separate header-file\n" 
+           "  -p               keep path when generating component name from filename\n"
            "  -L               disable generation of #line-directives\n";
       msg = o.str();
     }
@@ -333,6 +340,7 @@ int main(int argc, char* argv[])
 
   try
   {
+    log_init();
     tnt::ecppc::Ecppc app(argc, argv);
     return app.run();
   }
