@@ -1,20 +1,29 @@
-/* generator.cpp
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -22,7 +31,8 @@
 #include <tnt/stringescaper.h>
 #include <iterator>
 #include <zlib.h>
-#include <cxxtools/dynbuffer.h>
+#include <vector>
+#include <algorithm>
 #include <cxxtools/log.h>
 #include <tnt/httpmessage.h>
 #include <tnt/componentfactory.h>
@@ -33,14 +43,58 @@ namespace tnt
 {
   namespace ecppc
   {
+    namespace
+    {
+      class BStringPrinter
+      {
+          std::ostream& out;
+          unsigned mincol, maxcol, col;
+
+        public:
+          explicit BStringPrinter(std::ostream& out_, unsigned mincol_,
+            unsigned maxcol_, unsigned col_)
+            : out(out_),
+              mincol(mincol_),
+              maxcol(maxcol_),
+              col(col_)
+            { }
+
+          void print(const char* data, unsigned count);
+          void print(char ch);
+      };
+
+      void BStringPrinter::print(const char* data, unsigned count)
+      {
+        for (unsigned n = 0; n < count; ++n)
+          print(data[n]);
+      }
+
+      void BStringPrinter::print(char ch)
+      {
+        stringescaper s;
+        const char* d = s(ch);
+        unsigned len = strlen(d);
+        col += len;
+        if (col >= maxcol)
+        {
+          out << "\"\n";
+          for (unsigned n = 0; n < mincol; ++n)
+            out << ' ';
+          out << '"';
+          col = mincol + len;
+        }
+        out << d;
+      }
+    }
+
     ////////////////////////////////////////////////////////////////////////
     // Generator
     //
-    Generator::Generator(const std::string& classname)
+    Generator::Generator(const std::string& componentName)
       : raw(false),
-        maincomp(classname),
+        maincomp(componentName),
         haveCloseComp(false),
-        closeComp(classname),
+        closeComp(componentName),
         currentComp(&maincomp),
         externData(false),
         compress(false),
@@ -255,16 +309,16 @@ namespace tnt
       out << "////////////////////////////////////////////////////////////////////////\n"
              "// " << filename << "\n"
              "// generated with ecppc\n"
-             "// date: " << gentime <<
              "//\n\n";
     }
 
     void Generator::getHeaderIncludes(std::ostream& out) const
     {
-      out << "#include <tnt/ecpp.h>\n"
-             "#include <tnt/convert.h>\n";
-      if (!componentclass.empty())
-        out << "#include \"" << componentclass << ".h\"\n";
+      if (multiImages.empty() && !isRawMode())
+        out << "#include <tnt/ecpp.h>\n"
+               "#include <tnt/convert.h>\n";
+      else
+        out << "#include <tnt/mbcomponent.h>\n";
     }
 
     void Generator::getPre(std::ostream& out) const
@@ -282,24 +336,35 @@ namespace tnt
 
     void Generator::getNamespaceEnd(std::ostream& out) const
     {
-      out << "} // namespace\n\n";
+      out << "} // namespace\n";
     }
 
     void Generator::getClassDeclaration(std::ostream& out) const
     {
-      out << "class _component_" << maincomp.getName() << " : public ";
-      if (componentclass.empty())
-        out << "tnt::EcppComponent";
+      if (multiImages.empty() && !isRawMode())
+      {
+        out << "class _component_ : public tnt::EcppComponent\n"
+               "{\n"
+               "    _component_& main()  { return *this; }\n\n" 
+               "  protected:\n"
+               "    ~_component_();\n\n"
+               "  public:\n"
+               "    _component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);\n\n"
+               "    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);\n";
+      }
       else
-        out << componentclass;
-      out << "\n"
-             "{\n"
-             "    _component_" << maincomp.getName() << "& main()  { return *this; }\n\n" 
-             "  protected:\n"
-             "    ~_component_" << maincomp.getName() << "();\n\n"
-             "  public:\n"
-             "    _component_" << maincomp.getName() << "(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);\n\n"
-             "    unsigned operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam);\n";
+      {
+        out << "class _component_ : public tnt::MbComponent\n"
+               "{\n"
+               "    _component_& main()  { return *this; }\n\n" ;
+        if (compress)
+          out << "    tnt::DataChunks data;\n\n";
+        out << "  protected:\n"
+               "    ~_component_();\n\n"
+               "  public:\n"
+               "    _component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl);\n\n";
+      }
+
       if (haveCloseComp)
         out << "    unsigned endTag(tnt::HttpRequest& request, tnt::HttpReply& reply,\n"
                "                    tnt::QueryParams& qparam);\n";
@@ -345,88 +410,81 @@ namespace tnt
 
       out << "#include <cxxtools/log.h>\n"
              "#include <stdexcept>\n\n";
-
-      if (!multiImages.empty())
-        out << "#include <string.h>\n";
     }
 
     void Generator::getFactoryDeclaration(std::ostream& code) const
     {
       if (configs.empty())
       {
-        code << "static tnt::ComponentFactoryImpl<_component_" << maincomp.getName() << "> "
-             << maincomp.getName() << "Factory(\"" << maincomp.getName() << "\");\n\n";
+        code << "static tnt::ComponentFactoryImpl<_component_> Factory(\"" << maincomp.getName() << "\");\n\n";
       }
       else
       {
-        code << "class _component_" << maincomp.getName() << "Factory : public tnt::ComponentFactoryImpl<_component_" << maincomp.getName() << ">\n"
+        code << "class _component_Factory : public tnt::ComponentFactoryImpl<_component_>\n"
                 "{\n"
                 "  public:\n"
-                "    _component_" << maincomp.getName() << "Factory()\n"
-                "      : tnt::ComponentFactoryImpl<_component_" << maincomp.getName() << ">(\"" << maincomp.getName() << "\")\n"
+                "    _component_Factory()\n"
+                "      : tnt::ComponentFactoryImpl<_component_>(\"" << maincomp.getName() << "\")\n"
                 "      { }\n"
                 "    tnt::Component* doCreate(const tnt::Compident& ci,\n"
                 "      const tnt::Urlmapper& um, tnt::Comploader& cl);\n"
                 "    virtual void doConfigure(const tnt::Tntconfig& config);\n"
                 "};\n\n"
-                "tnt::Component* _component_" << maincomp.getName() << "Factory::doCreate(const tnt::Compident& ci,\n"
+                "tnt::Component* _component_Factory::doCreate(const tnt::Compident& ci,\n"
                 "  const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
                 "{\n"
-                "  return new _component_" << maincomp.getName() << "(ci, um, cl);\n"
+                "  return new _component_(ci, um, cl);\n"
                 "}\n\n"
-                "void _component_" << maincomp.getName() << "Factory::doConfigure(const tnt::Tntconfig& config)\n"
+                "void _component_Factory::doConfigure(const tnt::Tntconfig& config)\n"
                 "{\n"
                 "  // <%config>\n";
         for (variable_declarations::const_iterator it = configs.begin();
              it != configs.end(); ++it)
-          it->getConfigInit(code, "_component_" + maincomp.getName());
+          it->getConfigInit(code);
         code << "  // </%config>\n"
                 "}\n\n"
-                "static _component_" << maincomp.getName() << "Factory factory;\n\n";
+                "static _component_Factory factory;\n\n";
       }
     }
 
     void Generator::getCppBody(std::ostream& code) const
     {
-      std::string classname = "_component_" + maincomp.getName();
-
       getFactoryDeclaration(code);
 
-      if (compress)
+      if (!data.empty())
       {
-        code << "static tnt::Zdata rawData(\n\"";
-
-        uLongf s = data.size() * data.size() / 100 + 100;
-        cxxtools::Dynbuffer<Bytef> p;
-        p.reserve(s);
-
-        int z_ret = ::compress(p.data(), &s, (const Bytef*)data.ptr(), data.size());
-
-        if (z_ret != Z_OK)
+        if (compress)
         {
-          throw std::runtime_error(std::string("error compressing data: ") +
-            (z_ret == Z_MEM_ERROR ? "Z_MEM_ERROR" :
-             z_ret == Z_BUF_ERROR ? "Z_BUF_ERROR" :
-             z_ret == Z_DATA_ERROR ? "Z_DATA_ERROR" : "unknown error"));
+          code << "static tnt::Zdata rawData(\n\"";
+
+          uLongf s = data.size() + data.size() / 100 + 100;
+          std::vector<char> p(s);
+          char* buffer = &p[0];
+
+          int z_ret = ::compress(reinterpret_cast<Bytef*>(buffer), &s, (const Bytef*)data.ptr(), data.size());
+
+          if (z_ret != Z_OK)
+          {
+            throw std::runtime_error(std::string("error compressing data: ") +
+              (z_ret == Z_MEM_ERROR ? "Z_MEM_ERROR" :
+               z_ret == Z_BUF_ERROR ? "Z_BUF_ERROR" :
+               z_ret == Z_DATA_ERROR ? "Z_DATA_ERROR" : "unknown error"));
+          }
+
+          BStringPrinter bs(code, 2, 120, 30);
+          bs.print(buffer, s);
+
+          code << "\",\n  " << s << ", " << data.size() << ");\n\n";
         }
+        else
+        {
+          code << "static const char* rawData = \"";
 
-        std::transform(p.data(), p.data() + s,
-          std::ostream_iterator<const char*>(code),
-          stringescaper());
+          BStringPrinter bs(code, 2, 120, 30);
+          bs.print(data.ptr(), data.size());
 
-        code << "\",\n  " << s << ", " << data.size() << ");\n";
-      }
-      else
-      {
-        code << "static const char* rawData = \"";
-
-        std::transform(
-          data.ptr(),
-          data.ptr() + data.size(),
-          std::ostream_iterator<const char*>(code),
-          stringescaper());
-
-        code << "\";\n\n";
+          code << "\";\n\n";
+        }
       }
 
       // multi-images
@@ -446,155 +504,197 @@ namespace tnt
           code << "  \"" << it->mime << "\",\n";
 
         code << "};\n\n"
-                "const char* url_c_time[] = {\n";
+                "const char* ctimes[] = {\n";
 
         for (MultiImagesType::const_iterator it = multiImages.begin();
              it != multiImages.end(); ++it)
           code << "  \"" << tnt::HttpMessage::htdate(it->c_time) << "\",\n";
 
-        code << "};\n\n"
-                "typedef const char** urls_iterator;\n"
-                "static urls_iterator urls_begin = urls;\n"
-                "static urls_iterator urls_end = urls_begin + " << multiImages.size() << ";\n\n"
-                "inline bool charpLess(const char* a, const char* b)\n"
-                "{\n"
-                "  return strcmp(a, b) < 0;\n"
-                "}\n\n";
-      }
+        code << "};\n\n";
 
-      // logger, %shared and constructor
-      //
-      code << "// <%shared>\n"
-           << shared
-           << "// </%shared>\n\n";
-      if (!configs.empty())
-        code << "bool config_init = false;\n";
-      code << "// <%config>\n";
-      for (variable_declarations::const_iterator it = configs.begin();
-           it != configs.end(); ++it)
-        it->getConfigDecl(code, classname);
-      code << "// </%config>\n\n"
-              "#define SET_LANG(lang) \\\n"
-              "     do \\\n"
-              "     { \\\n"
-              "       request.setLang(lang); \\\n"
-              "       reply.setLocale(request.getLocale()); \\\n";
-      if (externData)
-        code << "       data.setData(getData(request, rawData)); \\\n";
-      code << "     } while (false)\n\n"
-           << classname << "::" << classname << "(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
-              "  : EcppComponent(ci, um, cl)";
-
-      // initialize subcomponents
-      for (subcomps_type::const_iterator i = subcomps.begin(); i != subcomps.end(); ++i)
-        code << ",\n"
-                "    " << i->getName() << "(*this, \"" << i->getName() << "\")";
-
-      code << "\n{\n";
-
-      if (compress)
-        code << "  rawData.addRef();\n";
-
-      code << "  // <%init>\n"
-           << init
-           << "  // </%init>\n"
-              "}\n\n"
-           << classname << "::~" << classname << "()\n"
-              "{\n"
-              "  // <%cleanup>\n"
-           << cleanup
-           << "  // </%cleanup>\n";
-
-      if (compress)
-        code << "  rawData.release();\n";
-
-      code << "}\n\n"
-              "unsigned " << classname << "::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)\n"
-           << "{\n";
-
-      if (isDebug())
-        code << "  log_trace(\"" << classname << " \" << qparam.getUrl());\n\n";
-
-      if (raw)
-        code << "  reply.setKeepAliveHeader();\n\n";
-      if (!mimetype.empty())
-        code << "  reply.setContentType(\"" << mimetype << "\");\n";
-
-      if (c_time)
-        code << "  {\n"
-                "    std::string s = request.getHeader(tnt::httpheader::ifModifiedSince);\n"
-                "    if (s == \"" << tnt::HttpMessage::htdate(c_time) << "\")\n"
-                "      return HTTP_NOT_MODIFIED;\n"
-                "  }\n";
-
-      if (!data.empty())
-      {
-        if (externData)
-          code << "  tnt::DataChunks data(getData(request, rawData));\n";
+        if (compress)
+        {
+          code << "_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
+                  "  : MbComponent(ci, um, cl)\n"
+                  "{\n"
+                  "  ::rawData.addRef();\n"
+                  "  data.setData(::rawData);\n"
+                  "  init(::rawData, ::urls, ::mimetypes, ::ctimes);\n"
+                  "}\n\n"
+                  "_component_::~_component_()\n"
+                  "{\n"
+                  "  ::rawData.release();\n"
+                  "}\n\n";
+        }
         else
-          code << "  tnt::DataChunks data(rawData);\n";
+        {
+          code << "_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
+                  "  : MbComponent(ci, um, cl, ::rawData, ::urls, ::mimetypes, ::ctimes)\n"
+                  "{ }\n\n"
+                  "_component_::~_component_()\n"
+                  "{ }\n\n";
+        }
       }
-
-      if (multiImages.empty())
+      else if (isRawMode())
       {
-        if (c_time)
-          code << "  reply.setHeader(tnt::httpheader::lastModified, \""
-               << tnt::HttpMessage::htdate(c_time) << "\");\n";
-        if (raw)
-          code << "  reply.setContentLengthHeader(data.size(0));\n"
-                  "  reply.setDirectMode();\n";
+        code << "static const char* mimetype = \"" << mimetype << "\";\n"
+                "static const char* c_time = \"" << tnt::HttpMessage::htdate(c_time) << "\";\n\n";
 
-        code << '\n';
-        maincomp.getBody(code, linenumbersEnabled);
-        code << "}\n\n";
+        if (compress)
+        {
+          code << "_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
+                  "  : MbComponent(ci, um, cl)\n"
+                  "{\n"
+                  "  ::rawData.addRef();\n"
+                  "  data.setData(::rawData);\n"
+                  "  init(::rawData, ::mimetype, ::c_time);\n"
+                  "}\n\n"
+                  "_component_::~_component_()\n"
+                  "{\n"
+                  "  ::rawData.release();\n"
+                  "}\n\n";
+        }
+        else
+        {
+          code << "_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
+                  "  : MbComponent(ci, um, cl, ::rawData, ::mimetype, ::c_time)\n"
+                  "{ }\n\n"
+                  "_component_::~_component_()\n"
+                  "{ }\n\n";
+        }
       }
       else
       {
-        // multi-image-component
-        code << "  const char* url = request.getPathInfo().c_str();\n\n"
-                "  log_debug(\"search for \\\"\" << url << '\"');\n\n"
-                "  urls_iterator it = std::lower_bound(urls_begin, urls_end, url, charpLess);\n"
-                "  if (it == urls_end || strcmp(url, *it) != 0)\n"
-                "  {\n"
-                "    log_info(\"binary file \\\"\" << url << \"\\\" not found\");\n"
-                "    return DECLINED;\n"
-                "  }\n"
-                "  unsigned url_idx = it - urls_begin;\n\n"
+        // logger, %shared and constructor
+        //
+        code << "// <%shared>\n"
+             << shared
+             << "// </%shared>\n\n";
+        code << "// <%config>\n";
+        for (variable_declarations::const_iterator it = configs.begin();
+             it != configs.end(); ++it)
+          it->getConfigDecl(code);
+        code << "// </%config>\n\n"
+                "#define SET_LANG(lang) \\\n"
+                "     do \\\n"
+                "     { \\\n"
+                "       request.setLang(lang); \\\n"
+                "       reply.setLocale(request.getLocale()); \\\n";
+        if (externData)
+          code << "       data.setData(getData(request, rawData)); \\\n";
+        code << "     } while (false)\n\n"
+             << "_component_::_component_(const tnt::Compident& ci, const tnt::Urlmapper& um, tnt::Comploader& cl)\n"
+                "  : EcppComponent(ci, um, cl)";
 
-                "  reply.setKeepAliveHeader();\n"
-                "  reply.setContentType(mimetypes[url_idx]);\n\n"
+        // initialize subcomponents
+        for (subcomps_type::const_iterator i = subcomps.begin(); i != subcomps.end(); ++i)
+          code << ",\n"
+                  "    " << i->getName() << "(*this, \"" << i->getName() << "\")";
 
-                "  std::string s = request.getHeader(tnt::httpheader::ifModifiedSince);\n"
-                "  if (s == url_c_time[url_idx])\n"
-                "    return HTTP_NOT_MODIFIED;\n\n"
+        code << "\n{\n";
 
-                "  reply.setHeader(tnt::httpheader::lastModified, url_c_time[url_idx]);\n"
-                "  reply.setContentLengthHeader(data.size(url_idx));\n"
-                "  reply.setDirectMode();\n"
-                "  reply.out() << data[url_idx];\n"
-                "  return HTTP_OK;\n"
-                "}\n\n";
+        if (compress)
+          code << "  rawData.addRef();\n";
+
+        code << "  // <%init>\n"
+             << init
+             << "  // </%init>\n"
+                "}\n\n"
+                "_component_::~_component_()\n"
+                "{\n"
+                "  // <%cleanup>\n"
+             << cleanup
+             << "  // </%cleanup>\n";
+
+        if (compress)
+          code << "  rawData.release();\n";
+
+        code << "}\n\n"
+                "unsigned _component_::operator() (tnt::HttpRequest& request, tnt::HttpReply& reply, tnt::QueryParams& qparam)\n"
+             << "{\n"
+                "  log_trace(\"" << maincomp.getName() << " \" << qparam.getUrl());\n\n";
+
+        if (raw)
+          code << "  reply.setKeepAliveHeader();\n\n";
+        if (!mimetype.empty())
+          code << "  reply.setContentType(\"" << mimetype << "\");\n";
+
+        if (c_time)
+          code << "  {\n"
+                  "    std::string s = request.getHeader(tnt::httpheader::ifModifiedSince);\n"
+                  "    if (s == \"" << tnt::HttpMessage::htdate(c_time) << "\")\n"
+                  "      return HTTP_NOT_MODIFIED;\n"
+                  "  }\n";
+
+        if (!data.empty())
+        {
+          if (externData)
+            code << "  tnt::DataChunks data(getData(request, rawData));\n";
+          else
+            code << "  tnt::DataChunks data(rawData);\n";
+        }
+
+        if (multiImages.empty())
+        {
+          if (c_time)
+            code << "  reply.setHeader(tnt::httpheader::lastModified, \""
+                 << tnt::HttpMessage::htdate(c_time) << "\");\n";
+          if (raw)
+            code << "  reply.setContentLengthHeader(data.size(0));\n"
+                    "  reply.setDirectMode();\n";
+
+          code << '\n';
+          maincomp.getBody(code, linenumbersEnabled);
+          code << "}\n\n";
+        }
+        else
+        {
+          // multi-image-component
+          code << "  const char* url = request.getPathInfo().c_str();\n\n"
+                  "  log_debug(\"search for \\\"\" << url << '\"');\n\n"
+                  "  urls_iterator it = std::lower_bound(urls_begin, urls_end, url, charpLess);\n"
+                  "  if (it == urls_end || strcmp(url, *it) != 0)\n"
+                  "  {\n"
+                  "    log_info(\"binary file \\\"\" << url << \"\\\" not found\");\n"
+                  "    return DECLINED;\n"
+                  "  }\n"
+                  "  unsigned url_idx = it - urls_begin;\n\n"
+
+                  "  reply.setKeepAliveHeader();\n"
+                  "  reply.setContentType(mimetypes[url_idx]);\n\n"
+
+                  "  std::string s = request.getHeader(tnt::httpheader::ifModifiedSince);\n"
+                  "  if (s == url_c_time[url_idx])\n"
+                  "    return HTTP_NOT_MODIFIED;\n\n"
+
+                  "  reply.setHeader(tnt::httpheader::lastModified, url_c_time[url_idx]);\n"
+                  "  reply.setContentLengthHeader(data.size(url_idx));\n"
+                  "  reply.setDirectMode();\n"
+                  "  reply.out() << data[url_idx];\n"
+                  "  return HTTP_OK;\n"
+                  "}\n\n";
+        }
+
+        if (haveCloseComp)
+          closeComp.getDefinition(code, externData, linenumbersEnabled);
+
+        if (!attr.empty())
+        {
+          code << "// <%attr>\n"
+                  "std::string _component_::getAttribute(const std::string& name, const std::string& def) const\n"
+                  "{\n";
+          for (attr_type::const_iterator it = attr.begin();
+               it != attr.end(); ++it)
+            code << "  if (name == \"" << it->first << "\")\n"
+                    "    return " << it->second << ";\n";
+          code << "  return def;\n"
+                  "} // </%attr>\n\n";
+        }
+
+        for (subcomps_type::const_iterator i = subcomps.begin();
+             i != subcomps.end(); ++i)
+          i->getDefinition(code, externData, linenumbersEnabled);
       }
-
-      if (haveCloseComp)
-        closeComp.getDefinition(code, externData, linenumbersEnabled);
-
-      if (!attr.empty())
-      {
-        code << "// <%attr>\n"
-                "std::string " << classname << "::getAttribute(const std::string& name, const std::string& def) const\n"
-                "{\n";
-        for (attr_type::const_iterator it = attr.begin();
-             it != attr.end(); ++it)
-          code << "  if (name == \"" << it->first << "\")\n"
-                  "    return " << it->second << ";\n";
-        code << "  return def;\n"
-                "} // </%attr>\n\n";
-      }
-
-      for (subcomps_type::const_iterator i = subcomps.begin();
-           i != subcomps.end(); ++i)
-        i->getDefinition(code, isDebug(), externData, linenumbersEnabled);
     }
 
     void Generator::printLine(std::ostream& out) const
@@ -603,58 +703,20 @@ namespace tnt
         out << "#line " << (curline + 1) << " \"" << curfile << "\"\n";
     }
 
-    void Generator::getHeader(std::ostream& header, const std::string& filename) const
-    {
-      std::string classname = "_component_" + maincomp.getName();
-
-      getIntro(header, filename);
-      header << "#ifndef ECPP_COMPONENT_" << maincomp.getName() << "_H\n"
-                "#define ECPP_COMPONENT_" << maincomp.getName() << "_H\n\n";
-
-      getHeaderIncludes(header);
-      header << '\n';
-      getPre(header);
-      header << '\n';
-      getNamespaceStart(header);
-      getClassDeclaration(header);
-      getNamespaceEnd(header);
-
-      header << "\n"
-                "#endif\n";
-    }
-
     void Generator::getCpp(std::ostream& code, const std::string& filename) const
-    {
-      getIntro(code, filename);
-      getCppIncludes(code);
-
-      code << "#include \"" << maincomp.getName() << ".h\"\n\n"
-              "log_define(\"component." << maincomp.getName() << "\")\n\n";
-
-      getNamespaceStart(code);
-
-      code << "template <typename T> inline void use(const T&) { }\n\n";
-
-      getCppBody(code);
-      getNamespaceEnd(code);
-    }
-
-    void Generator::getCppWoHeader(std::ostream& code, const std::string& filename) const
     {
       getIntro(code, filename);
 
       getHeaderIncludes(code);
       getCppIncludes(code);
 
-      code << "log_define(\"component." << maincomp.getName() << "\")\n\n";
+      code << "log_define(\"component." << maincomp.getLogCategory() << "\")\n\n";
 
       getPre(code);
       code << '\n';
 
       getNamespaceStart(code);
 
-      code << "template <typename T> inline void use(const T&) { }\n\n";
-
       getClassDeclaration(code);
 
       getCppBody(code);
