@@ -1,20 +1,29 @@
-/* generator.h
+/*
  * Copyright (C) 2003-2005 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
+ * 
+ * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
+ * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 
@@ -42,10 +51,8 @@ namespace tnt
     //
     class Generator : public tnt::ecpp::ParseHandler
     {
-        bool debug;
         bool raw;
         std::string mimetype;
-        std::string componentclass;
 
         typedef std::list<tnt::ecppc::Variable> variable_declarations;
 
@@ -109,17 +116,11 @@ namespace tnt
         void printLine(std::ostream& out) const;
 
       public:
-        Generator(const std::string& classname);
-
-        void setDebug(bool sw)                       { debug = sw; }
-        bool isDebug() const                         { return debug; }
+        Generator(const std::string& componentName);
 
         void setMimetype(const std::string& type)    { mimetype = type; }
         const std::string& getMimetype()             { return mimetype; }
 
-        void setComponentclass(const std::string& c) { componentclass = c; }
-        const std::string& getComponentclass()       { return componentclass; }
-
         void setRawMode(bool sw = true)              { raw = sw; }
         bool isRawMode() const                       { return raw; }
 
@@ -165,9 +166,7 @@ namespace tnt
         virtual void startI18n();
         virtual void endI18n();
 
-        void getHeader(std::ostream& out, const std::string& filename) const;
         void getCpp(std::ostream& out, const std::string& filename) const;
-        void getCppWoHeader(std::ostream& out, const std::string& filename) const;
     };
 
   }
